/*
 *   Copyright 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spago4Q.

    Spago4Q is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published 
    by the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spago4Q is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package it.eng.extractors.svn;

import java.io.ByteArrayOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.tmatesoft.svn.core.SVNDirEntry;
import org.tmatesoft.svn.core.SVNException;
import org.tmatesoft.svn.core.SVNLogEntry;
import org.tmatesoft.svn.core.SVNLogEntryPath;
import org.tmatesoft.svn.core.SVNNodeKind;
import org.tmatesoft.svn.core.SVNProperty;
import org.tmatesoft.svn.core.SVNURL;
import org.tmatesoft.svn.core.auth.BasicAuthenticationManager;
import org.tmatesoft.svn.core.auth.ISVNAuthenticationManager;
import org.tmatesoft.svn.core.internal.io.dav.DAVRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.fs.FSRepositoryFactory;
import org.tmatesoft.svn.core.internal.io.svn.SVNRepositoryFactoryImpl;
import org.tmatesoft.svn.core.io.SVNRepository;
import org.tmatesoft.svn.core.io.SVNRepositoryFactory;
import org.tmatesoft.svn.core.wc.SVNWCUtil;

public class SvnBLogic {

	/*
	 * Initializes the library to work with a repository via different
	 * protocols.
	 */
	private void setupLibrary() {
		/*
		 * For using over http:// and https://
		 */
		DAVRepositoryFactory.setup();
		/*
		 * For using over svn:// and svn+xxx://
		 */
		SVNRepositoryFactoryImpl.setup();

		/*
		 * For using over file:///
		 */
		FSRepositoryFactory.setup();
	}

	/*
	 * args parameter is used to obtain a repository location URL, user's
	 * account name & password to authenticate him to the server.
	 */
	public SVNRepository getRepository(String url, String name, String password)
			throws SVNException {

		/*
		 * initializes the library (it must be done before ever using the
		 * library itself)
		 */
		setupLibrary();

		SVNRepository repository = null;
		/*
		 * Creates an instance of SVNRepository to work with the repository. All
		 * user's requests to the repository are relative to the repository
		 * location used to create this SVNRepository. SVNURL is a wrapper for
		 * URL strings that refer to repository locations.
		 */
		repository = SVNRepositoryFactory.create(SVNURL.parseURIEncoded(url));

		/*
		 * User's authentication information (name/password) is provided via an
		 * ISVNAuthenticationManager instance. SVNWCUtil creates a default
		 * authentication manager given user's name and password.
		 * 
		 * Default authentication manager first attempts to use provided user
		 * name and password and then falls back to the credentials stored in
		 * the default Subversion credentials storage that is located in
		 * Subversion configuration area. If you'd like to use provided user
		 * name and password only you may use BasicAuthenticationManager class
		 * instead of default authentication manager:
		 * 
		 * authManager = new BasicAuthenticationsManager(userName,
		 * userPassword);
		 * 
		 * You may also skip this point - anonymous access will be used.
		 */
		ISVNAuthenticationManager authManager = SVNWCUtil
				.createDefaultAuthenticationManager(name, password);

		repository.setAuthenticationManager(authManager);
		
		/**
		 * in case of proxy, use the basicAuthenticationManager 
		 * instead of the DefaultAuthenticationManager
		 * note: remove the previous setAuthenticationManager line 
		 */
		/*
		String proxyUlr = "";
		int proxyPort = 0;
		String proxyUsr = "";
		String proxyPwd = "";
		BasicAuthenticationManager basicAuthenticationManager = new BasicAuthenticationManager(name, password);
		basicAuthenticationManager.setProxy(proxyUlr, proxyPort, proxyUsr, proxyPwd);
		repository.setAuthenticationManager(basicAuthenticationManager);
		*/
		
		return repository;

	}

	/*
	 * Called recursively to obtain all entries that make up the repository tree
	 * repository - an SVNRepository which interface is used to carry out the
	 * request, in this case it's a request to get all entries in the directory
	 * located at the path parameter;
	 * 
	 * path is a directory path relative to the repository location path (that
	 * is a part of the URL used to create an SVNRepository instance);
	 * 
	 * This example shows how to get the repository tree at the latest (HEAD)
	 * revision starting with the directory that is the path/to/repository part
	 * of the repository location URL. The main point is SVNRepository.getDir()
	 * method that is called recursively for each directory (till the end of the
	 * tree). getDir collects all entries located inside a directory and returns
	 * them as a java.util.Collection. As an example here's one of the program
	 * layouts (for the default url used in the program ):
	 */
	public void listEntries(SVNRepository repository, String path, long revision)
			throws SVNException {
		/*
		 * Gets the contents of the directory specified by path at the latest
		 * revision (for this purpose -1 is used here as the revision number to
		 * mean HEAD-revision) getDir returns a Collection of SVNDirEntry
		 * elements. SVNDirEntry represents information about the directory
		 * entry. Here this information is used to get the entry name, the name
		 * of the person who last changed this entry, the number of the revision
		 * when it was last changed and the entry type to determine whether it's
		 * a directory or a file. If it's a directory listEntries steps into a
		 * next recursion to display the contents of this directory. The third
		 * parameter of getDir is null and means that a user is not interested
		 * in directory properties. The fourth one is null, too - the user
		 * doesn't provide its own Collection instance and uses the one returned
		 * by getDir.
		 */
		Collection entries = repository.getDir(path, revision, null,
				(Collection) null);
		Iterator iterator = entries.iterator();
		while (iterator.hasNext()) {
			SVNDirEntry entry = (SVNDirEntry) iterator.next();
			System.out.println("/" + (path.equals("") ? "" : path + "/")
					+ entry.getName() + " (author: '" + entry.getAuthor()
					+ "'; revision: " + entry.getRevision() + "; date: "
					+ entry.getDate() + ")");
			/*
			 * Checking up if the entry is a directory.
			 */
			if (entry.getKind() == SVNNodeKind.DIR) {
				listEntries(repository, (path.equals("")) ? entry.getName()
						: path + "/" + entry.getName(), revision);
			}
		}
	}

	/*
	 * stampa entries dell'albero scendendo di x livelli
	 */
	public void listEntries(SVNRepository repository, String path, long revision, int level)
			throws SVNException {

		int livello = level;
		Collection entries = repository.getDir(path, revision, null,(Collection) null);
		Iterator iterator = entries.iterator();
		while (iterator.hasNext()) {
			SVNDirEntry entry = (SVNDirEntry) iterator.next();
			System.out.println("/" + (path.equals("") ? "" : path + "/")
					+ entry.getName() + " (author: '" + entry.getAuthor()
					+ "'; revision: " + entry.getRevision() + "; date: "
					+ entry.getDate() + ")");
			/*
			 * Checking up if the entry is a directory.
			 */
			if ((entry.getKind() == SVNNodeKind.DIR) && (livello > 0)) {
				livello--;
				listEntries(repository, (path.equals("")) ? entry.getName()
						: path + "/" + entry.getName(), revision, livello);
			}
		}
	}
	
	/*
	 * popola lista entries dell'albero scendendo di x livelli
	 */

	public void getListEntries(
			Collection<SVNDirEntry> listaEntry,SVNRepository repository,
			String extensionFilter, String path, 
			long revision, int level) throws SVNException {

		int livello = level;
		
		String fileFilter = path.substring(path.lastIndexOf('/')+1);
		if (!"".equals(fileFilter)) {
			// path di un file
			if (fileFilter.endsWith("."+extensionFilter)) {
				// filename corretto
				path = path.substring(0,path.lastIndexOf('/'));	// ottendo path directory
			} else {
				fileFilter="";
			}
		}

		Collection entries = repository.getDir(path, revision, null,(Collection) null);
		Iterator iterator = entries.iterator();
		while (iterator.hasNext()) {
			SVNDirEntry entry = (SVNDirEntry) iterator.next();
			if (entry.getKind() == SVNNodeKind.FILE) {
				String ext = entry.getName().substring(entry.getName().lastIndexOf('.')+1);
				if ("".equals(fileFilter)) {
					// filtro estensione
					if (extensionFilter.equalsIgnoreCase(ext)) {
						listaEntry.add(entry);
					}
				} else {
					// filtro singolo file 
					if (fileFilter.equals(entry.getName())) {
						listaEntry.add(entry);
					}
				}
			}

			/*
			 * Checking up if the entry is a directory.
			 */
			if (entry.getKind() == SVNNodeKind.DIR) {
				if ((livello > 0) || (level==-1)) {
					livello--;
					getListEntries(listaEntry, repository, extensionFilter, (path.equals("")) ? entry.getName()
							: path + "/" + entry.getName(), revision, livello);
				}
			}
		} 
	}


	/*
	 * Get type of the node located at URL we used to create SVNRepository. ""
	 * (empty string) is path relative to that URL, -1 is value that may be used
	 * to specify HEAD (latest) revision.
	 */
	public SVNNodeKind getTypeNode(SVNRepository repository, String filePath,
			long revision) throws SVNException {

		SVNNodeKind nodeKind = repository.checkPath(filePath, revision);

		return nodeKind;
	}

	public Map getFileProperties(SVNRepository repository, String filePath,
			long revision) throws SVNException {
		/*
		 * This Map will be used to get the file properties. Each Map key is a
		 * property name and the value associated with the key is the property
		 * value.
		 */
		Map fileProperties = new HashMap();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		/*
		 * Gets the contents and properties of the file located at filePath in
		 * the repository at the latest revision (which is meant by a negative
		 * revision number).
		 */
		repository.getFile(filePath, revision, fileProperties, baos);

		return fileProperties;
	}

	public ByteArrayOutputStream getFile(SVNRepository repository,
			String filePath, long revision) throws SVNException {
		/*
		 * This Map will be used to get the file properties. Each Map key is a
		 * property name and the value associated with the key is the property
		 * value.
		 */
		Map fileProperties = new HashMap();
		ByteArrayOutputStream baos = new ByteArrayOutputStream();

		/*
		 * Gets the contents and properties of the file located at filePath in
		 * the repository at the latest revision (which is meant by a negative
		 * revision number).
		 */
//		System.out.println("getLocation : " + repository.getLocation().toString());
//		System.out.println("getHost : " + repository.getRepositoryRoot().getHost());
//		System.out.println("getPath : " + repository.getRepositoryRoot().getPath());
//		System.out.println("getURIEncodedPath : " + repository.getRepositoryRoot().getURIEncodedPath());
//				
		repository.getFile(filePath, revision, fileProperties, baos);

		return baos;
	}

	public void printPropertiesFile(Map fileProperties) {

		Iterator iterator = fileProperties.keySet().iterator();
		/*
		 * Displays file properties.
		 */
		while (iterator.hasNext()) {
			String propertyName = (String) iterator.next();
			String propertyValue = (String) fileProperties.get(propertyName);
			System.out.println("File property: " + propertyName + "=" + propertyValue);
		}
	}

	public void printFile(Map fileProperties, ByteArrayOutputStream baos) {
		/*
		 * Here the SVNProperty class is used to get the value of the
		 * svn:mime-type property (if any). SVNProperty is used to facilitate
		 * the work with versioned properties.
		 */

		String mimeType = (String) fileProperties.get(SVNProperty.MIME_TYPE);
		/*
		 * SVNProperty.isTextMimeType(..) method checks up the value of the
		 * mime-type file property and says if the file is a text (true) or not
		 * (false).
		 */
		boolean isTextType = SVNProperty.isTextMimeType(mimeType);

		/*
		 * Displays the file contents in the console if the file is a text.
		 */
		if (isTextType) {
			System.out.println("File contents:");
			System.out.println();
			try {
				baos.writeTo(System.out);
			} catch (IOException ioe) {
				ioe.printStackTrace();
			}
		} else {
			System.out.println("File contents can not be displayed in the console since the mime-type property says that it's not a kind of a text file.");
		}

	}

	public Collection getLogEntries(SVNRepository repository,
			long startRevision, long endRevision) throws SVNException {

		Collection logEntries = null;
		/*
		 * Collects SVNLogEntry objects for all revisions in the range defined
		 * by its start and end points [startRevision, endRevision]. For each
		 * revision commit information is represented by SVNLogEntry.
		 * 
		 * the 1st parameter (targetPaths - an array of path strings) is set
		 * when restricting the [startRevision, endRevision] range to only those
		 * revisions when the paths in targetPaths were changed.
		 * 
		 * the 2nd parameter if non-null - is a user's Collection that will be
		 * filled up with found SVNLogEntry objects; it's just another way to
		 * reach the scope.
		 * 
		 * startRevision, endRevision - to define a range of revisions you are
		 * interested in; by default in this program - startRevision=0,
		 * endRevision= the latest (HEAD) revision of the repository.
		 * 
		 * the 5th parameter - a boolean flag changedPath - if true then for
		 * each revision a corresponding SVNLogEntry will contain a map of all
		 * paths which were changed in that revision.
		 * 
		 * the 6th parameter - a boolean flag strictNode - if false and a
		 * changed path is a copy (branch) of an existing one in the repository
		 * then the history for its origin will be traversed; it means the
		 * history of changes of the target URL (and all that there's in that
		 * URL) will include the history of the origin path(s). Otherwise if
		 * strictNode is true then the origin path history won't be included.
		 * 
		 * The return value is a Collection filled up with SVNLogEntry Objects.
		 */
		logEntries = repository.log(new String[] { "" }, null, startRevision,
				endRevision, true, true);

		return logEntries;
	}

	public void printLogEntries(Collection logEntries, String nomeFile) {

		StringBuffer sb = new StringBuffer();
		for (Iterator entries = logEntries.iterator(); entries.hasNext();) {
			/*
			 * gets a next SVNLogEntry
			 */
			SVNLogEntry logEntry = (SVNLogEntry) entries.next();
			sb.append("---------------------------------------------");
			sb.append("\n");
			/*
			 * gets the revision number
			 */
			sb.append("revision: " + logEntry.getRevision());
			sb.append("\n");
			/*
			 * gets the author of the changes made in that revision
			 */
			sb.append("author: " + logEntry.getAuthor());
			sb.append("\n");
			/*
			 * gets the time moment when the changes were committed
			 */
			sb.append("date: " + logEntry.getDate());
			sb.append("\n");
			/*
			 * gets the commit log message
			 */
			sb.append("log message: " + logEntry.getMessage());
			sb.append("\n");
			/*
			 * displaying all paths that were changed in that revision; cahnged
			 * path information is represented by SVNLogEntryPath.
			 */
			if (logEntry.getChangedPaths().size() > 0) {
				sb.append("\n");
				sb.append("changed paths:");
				/*
				 * keys are changed paths
				 */
				Set changedPathsSet = logEntry.getChangedPaths().keySet();
				for (Iterator changedPaths = changedPathsSet.iterator(); changedPaths
						.hasNext();) {
					/*
					 * obtains a next SVNLogEntryPath
					 */
					SVNLogEntryPath entryPath = (SVNLogEntryPath) logEntry
							.getChangedPaths().get(changedPaths.next());
					/*
					 * SVNLogEntryPath.getPath returns the changed path itself;
					 * 
					 * SVNLogEntryPath.getType returns a charecter describing
					 * how the path was changed ('A' - added, 'D' - deleted or
					 * 'M' - modified);
					 * 
					 * If the path was copied from another one (branched) then
					 * SVNLogEntryPath.getCopyPath &
					 * SVNLogEntryPath.getCopyRevision tells where it was copied
					 * from and what revision the origin path was at.
					 */
					sb.append(" "
							+ entryPath.getType()
							+ "	"
							+ entryPath.getPath()
							+ ((entryPath.getCopyPath() != null) ? " (from "
									+ entryPath.getCopyPath() + " revision "
									+ entryPath.getCopyRevision() + ")" : ""));
					sb.append("\n");
				}
			}
		}
		
		FileWriter out = null;
		try {
			out = new FileWriter(nomeFile);
			out.write(sb.toString());
			out.close();
		} catch (Exception e) {
			System.out.print(sb.toString());
		}

	}
}
